#include "AffineTriangle.h"
#include "Triangle.h"

/******************************************************************
				AFFINE TRIANGLE COORDS STUFF
******************************************************************/

//
//  For any point p in the triangle (v1,v2,v3), 
//  p = (1-s-t)*v1 + s*v2 + t*v3, for 0<=s,t<=1 .
//
//	Re-express this as p = v1 + s*(v2-v1) + t*(v3-v1).
// 
//  This can be expressed in matrix form.  The code below
//  computes the inverse of the coefficient matrix which
//  allows us to quickly compute the affine coordinates.
//
void AffineTriangle::setup( Triangle& tri )
{
	v1 = vec2( tri.screen(0), VZ );
	vec2 v2( tri.screen(1), VZ );
	vec2 v3( tri.screen(2), VZ );

	// compute edge vectors (v2-v1) and (v3-v1)
	v12 = vec2(v2,VZ)-vec2(v1,VZ);
	v13 = vec2(v3,VZ)-vec2(v1,VZ);

	// calculate inverse of the determinant
	double invDet = 1.0 / (v12[VX]*v13[VY] - v13[VX]*v12[VY]);

	// inverse matrix using (some math guy)'s Rule.
	Minv[0][0] = invDet * v13[VY];
	Minv[0][1] = -(invDet * v13[VX]);
	Minv[1][0] = -(invDet * v12[VY]);
	Minv[1][1] = invDet * v12[VX];
}

//
//  Compute affine coords by hitting the vector from v1 to 
//  the point of interest by the precomputed inverse matrix.
//
vec3 AffineTriangle::computeAC( vec2& point ) 
{
	vec2 pToPoint = point - v1;
	double s, t;

	s = Minv[0][0]*pToPoint[VX] + Minv[0][1]*pToPoint[VY];
	t = Minv[1][0]*pToPoint[VX] + Minv[1][1]*pToPoint[VY];

	return vec3( 1.0-s-t, s, t );
}
