#ifndef AffineTriangle_h
#define AffineTriangle_h

#include "algebra3.h"

class Triangle;

/****************************************************************

	Class:		AffineTriangle
	Purpose:	Class for quickly computing the affine coordinates
				of arbitrary points inside a particular screenspace
				triangle.

				Computation of affine coordinates amounts to solving
				a 2x2 linear system.  By pre-inverting the coefficient
				matrix for the given triangle, we can quickly compute
				affine coordinates for any point in the triangle.

*****************************************************************/
class AffineTriangle
{
public:
	AffineTriangle() {};

	// set up affine triangle based on 3 screenspace
	void setup( Triangle& t );

	// computes the affine coordinates of a particular point
	vec3 computeAC( vec2& point );

private:

	vec2 v12;
	vec2 v13;

	double Minv[2][2];
	vec2 v1;
};


#endif