#include "BSP.h"

BSPnode::BSPnode( const Face& face )
: f(face)
{
	children[0] = -1;
	children[1] = -1;
}

BSPtree::BSPtree( int numFaces ) 
: tree(numFaces)
{
}

void BSPtree::addInSubtree( Face& newFace, int root )
{
	Face posFace;
	Face negFace;
	int newIndex;

	while( true ) {
		BSPnode& rootNode = tree[root];
		Face& rootFace = rootNode.f;

		IsectType ist = newFace.split( rootFace.plane(), WORLD, posFace, negFace );

		if( ist == POS ) {

			if( rootNode.children[0] == -1 ) {

				newIndex = tree.size();
				tree.append( BSPnode( newFace ) );
				rootNode.children[0] = newIndex;
				break;

			} else {

				root = rootNode.children[0];
				continue;

			}

		} else if( ist == NEG ) {

			if( rootNode.children[1] == -1 ) {

				newIndex = tree.size();
				tree.append( BSPnode( newFace ) );
				rootNode.children[1] = newIndex;
				break;

			} else {

				root = rootNode.children[1];
				continue;
			}

		} else if( ist == BOTH ) {

			if( rootNode.children[0] == -1 ) {

				newIndex = tree.size();
				tree.append( BSPnode( posFace ) );
				rootNode.children[0] = newIndex;

			} else {

				addInSubtree( posFace, rootNode.children[0] );
			}

			if( rootNode.children[1] == -1 ) {

				newIndex = tree.size();
				tree.append( BSPnode( negFace ) );
				rootNode.children[1] = newIndex;

			} else {

				addInSubtree( negFace, rootNode.children[1] );

			}
			break;

		}
	}
}

void BSPtree::addFace( Face& newFace ) 
{
	if( tree.size() == 0 ) {
		tree.append( BSPnode( newFace ) );
	} else {
		addInSubtree( newFace, 0 );
	}
}

void BSPtree::moveSubtree( int subTreeRoot, BSPtree& newTree ) 
{
	if( subTreeRoot == -1 ) {
		return;

	} else {
		BSPnode& curNode = tree[subTreeRoot];
		newTree.addFace( curNode.f );
		moveSubtree( curNode.children[0], newTree );
		moveSubtree( curNode.children[1], newTree );
	}
}

Face& BSPtree::operator[]( int index )
{
	return tree[index].f;
}

void BSPtree::order( OrderType which, vec3& viewPoint, List<int>& tOrder )
{
	int numFaces = tree.size();

	if( numFaces > 0 ) {
		orderSubtree( which, viewPoint, tOrder, 0 );
	}
}

void BSPtree::orderSubtree( OrderType which, vec3& viewPoint, List<int>& traversalOrder, int subTreeRoot )
{
	if( subTreeRoot == -1 ) {
		
		return;
	
	} else {

		BSPnode& rootNode = tree[subTreeRoot];
		Face& rootFace = rootNode.face();
		SideType camSide = inside( viewPoint, rootFace.plane() );

		switch( camSide ) {

			case INPLANE:
			case INSIDE:

				if( camSide == INPLANE ) {
					cout << "IN PLANE!!! " << rootFace.plane() << endl;
				}

				if( which == BACKTOFRONT ) {
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[1] );
					traversalOrder.append( subTreeRoot );
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[0] );
				} else {
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[0] );
					traversalOrder.append( subTreeRoot );
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[1] );
				}

				break;

			case OUTSIDE:

				if( which == BACKTOFRONT ) {
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[0] );
					traversalOrder.append( subTreeRoot );
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[1] );
				} else {
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[1] );
					traversalOrder.append( subTreeRoot );
					orderSubtree( which, viewPoint, traversalOrder, rootNode.children[0] );
				}
				break;
		}

	}
}

