#ifndef BSP_h
#define BSP_h

#include "Face.h"
#include "List.h"

/* node in a BSP tree */
class BSPnode
{
public:
	BSPnode() {};
	BSPnode( const Face& face );

	Face face() { return f; }

	int child( int which ) { return children[which]; }

	friend class BSPtree;
	friend ostream& operator<<( ostream& s, BSPnode &bspNode ) {return s;};

private:
	Face f;
	int children[2];
};

typedef enum { FRONTTOBACK, BACKTOFRONT } OrderType;

class BSPtree
{
public:
	BSPtree( int numFaces );

	void order( OrderType which, vec3& viewPoint, List<int>& traversalOrder );

	Face& operator[]( int index );

	void addFace( Face& newFace );

	void moveSubtree( int index, BSPtree& otherTree );

	int size() { return tree.size(); }

private:
	
	void addInSubtree( Face& newFace, int subTreeRoot );
	void orderSubtree( OrderType which, vec3& viewPoint, List<int>& traversalOrder, int subTreeRoot );
	List<Face> faces;
	List<BSPnode> tree; 
};

#endif