#include "BSPoctree.h"
#include "BoundingBox.h"

const int maxNodeCount = 50;

BSPoctree::BSPoctree() 
: bspTree(NULL), polyCount(0), children(NULL), parent(NULL)
{
}

BSPoctree::~BSPoctree()
{
	if( bspTree ) {
		delete bspTree;
	}
}

BSPoctree::BSPoctree( const BoundingBox& box )
: polyCount(0), children(NULL), parent(NULL), bb(box)
{
	vec3 mid = bb.midBound();
	vec3 low = bb.lowBound();
	vec3 high = bb.highBound();

	bspTree = new BSPtree( maxNodeCount );

	Face axisPlane(4);
	axisPlane.resize(4);

	// yz plane
	
	axisPlane.world(0) = vec3( mid[VX], low[VY], high[VZ] );
	axisPlane.world(1) = vec3( mid[VX], low[VY], low[VZ] );
	axisPlane.world(2) = vec3( mid[VX], high[VY], low[VZ] );
	axisPlane.world(3) = vec3( mid[VX], high[VY], high[VZ] );
	axisPlane.plane() = vec4( 1.0, 0.0, 0.0, -mid[VX] );
	bspTree->addFace( axisPlane );

	// xy plane
	axisPlane.world(0) = vec3( low[VX], low[VY], mid[VZ] );
	axisPlane.world(1) = vec3( high[VX], low[VY], mid[VZ] );
	axisPlane.world(2) = vec3( high[VX], high[VY], mid[VZ] );
	axisPlane.world(3) = vec3( low[VX], high[VY], mid[VZ] );
	axisPlane.plane() = vec4( 0.0, 0.0, 1.0, -mid[VZ] );
	bspTree->addFace( axisPlane );

	// xz plane
	axisPlane.world(0) = vec3( low[VX], mid[VY], high[VZ] );
	axisPlane.world(1) = vec3( high[VX], mid[VY], high[VZ] );
	axisPlane.world(2) = vec3( high[VX], mid[VY], low[VZ] );
	axisPlane.world(3) = vec3( low[VX], mid[VY], low[VZ] );
	axisPlane.plane() = vec4( 0.0, 1.0, 0.0, -mid[VY] );
	bspTree->addFace( axisPlane );
}

void BSPoctree::insert( Face& face )
{
	Face newFace(3);

	if( children != NULL ) {

		// this node has been split, so distribute poly to children
		for( unsigned char octant = 0; octant < 8; octant++ ) {

			if( face.clipToBBoxOctant( bb, octant, WORLD, newFace ) != NEG ) {
				children[octant]->insert( newFace );
			}
		
		}

	} else if( polyCount >= maxNodeCount ) {

		// if too many polys, split this node
		split();

	} else {

		// insert poly into current BSP tree
		bspTree->addFace( face );
		
	}
}

void BSPoctree::split()
{
	children = new BSPoctree*[8];

	for( unsigned char octant = 0; octant < 8; octant++ ) {

		BoundingBox newBB = bb.subBox( octant );
		children[octant] = new BSPoctree( newBB );

		int subIndex = 10 - octant;

		bspTree->moveSubtree( subIndex, (*children[octant]->bspTree) );
	}
}
































