#ifndef BspTree_h
#define BspTree_h

#include "List.h"

class vec3;
class Tessellation;

/****************************************************************

	Class:		BspTree
	Purpose:	Used to organize a tessellation into a BSP tree, and
				provide front-to-back traversal orders from arbitrary
				viewpoints.  The BSP tree is constructed by adding
				faces to the tessellation.

*****************************************************************/
class BspTree
{
public:

	// initialize the BSP tree from the specified list of faces in
	// the given tessellation structure.
	BspTree( Tessellation *tess, List<int>& faceList );

	// inserts a face from the tessellation into the BSP tree
	void insertFace( int face, int nodeIndex = 0 );

	// compute a front-to-back face ordering from a particular viewpoint, 
	// starting at a particular node in the tree (node 0 is the root)
	void orderFaces( vec3& viewPoint, List<int>& orderedFaces, int node = 0 );

	// print the tree
	friend ostream& operator<<( ostream& s, BspTree& t );

private:

	// appends a leaf to the tree (not attached to anything, though)
	int addFace( int face );

	Tessellation *tess;		// the tessellation from which the tree is constructed
	List<int> nodes;		// encodes the tree - sequence of (face,left child, right child)
							// triples.
	int size;				// number of nodes in the tree
};

#endif