#ifndef Edge_h
#define Edge_h

#include "algebra3.h"

// enumeration represents pixel area covered by an edge
typedef enum { ALL, NONE, SOME } OverlapType;

/****************************************************************

	Class:		Edge
	Purpose:	Represents an edge of a poly.  Operations to intersect
				the edge with a square region of screenspace.

				We discretize the perimeter of the screenspace 
				squares as described in TriageMask.h.

*****************************************************************/
class Edge
{
public:
	Edge() {};
	Edge( vec3& v1, vec3& v2 );

	// intersects edge with the given 2d box, returns intersections with
	// box edge.  'correct' invokes a conservative intersection, 
	// rounding to the outside of the poly.
	OverlapType boxIntersect( vec4& box, int *coords, bool correct );

private:
	
	// determines whether two points on the perimeter of a square are
	// on the same side of the square
	OverlapType sameSide( int v1, int v2 );

	// gets the index of a point on the perimeter of a box
	int getIndex( vec4& box, vec2& vertex, int whichSide = -1 );

	vec2 v[2];			// vertices of the edge
	vec2 deltaV;		// delta v
	double dx;			// delta x
	double dy;			// delta y
	double slope;		// slope
	double invSlope;	// inverse slope
};


#endif