#ifndef EdgeTable_h
#define EdgeTable_h

#include "algebra3.h"
#include "Lerp.h"

class Triangle;
class EdgeTableEntry;


/****************************************************************

	Class:		EdgeTable
	Purpose:	Edge table for incremental scan conversion.	Easy to
				use, just add a triangle, call startTraversal(),
				then repeatedly call getCurrentSpan() to get successive
				scanlines for the triangle.

*****************************************************************/
class EdgeTable
{
public:

	// initialize table for a specified y range
	EdgeTable( vec2& yRange, bool debug );
	~EdgeTable();

	// add a triangle to the edge table
	void addTriangle( Triangle& tri );

	// begin traversing the table one scanline at a time in increasing
	// scanline order
	bool startTraversal();

	// retrieve information about the current scanline
	bool getCurrentSpan( int& scanline, vec2& xRange, LerpVec3& coords );

	// clear the edge table
	void clear();

	// diagnostic output
	friend ostream& operator<< ( ostream& s, EdgeTable &edgeTable );

private:

	EdgeTableEntry *table;			// 2 EdgeTableEntries per scanline
	int *tableIndices;				// indicates how many entries on each scanline
	int tableYMin, tableYMax;		// first and last scanlines with entries
	int numScanlines;				// how many scanlines
	EdgeTableEntry *activeEdges;	// active edge table
	int currentScanline;			// current scanline (duh)

	bool debug;						// debugging flag
};


#endif