#ifndef Face_h
#define Face_h

#include "algebra3.h"
#include "Triangle.h"
#include "List.h"
#include "BoundingBox.h"

class Face
{
public:
	Face( int numPoints = 3);
	Face( const Face& other );

	Face& operator=( const Face& rhs );

	int size() { return wPoints.size(); };

	vec3& point   (int i, SpaceType space );

	List<vec3>& world  () { return wPoints; };
	List<vec3>& camera () { return cPoints; };
	List<vec3>& local  () { return lPoints; };
	List<vec3>& screen () { return sPoints; };
	List<vec2>& texture() { return tPoints; };
	List<vec3>& normal () { return normals; };

	vec3& world   (int i ) { return wPoints[i]; };
	vec3& camera  (int i ) { return cPoints[i]; };
	vec3& local   (int i ) { return lPoints[i]; };
	vec3& screen  (int i ) { return sPoints[i]; };
	vec2& texture (int i ) { return tPoints[i]; };
	vec3& normal  (int i ) { return normals[i]; };
	vec4& plane   ()       { return pEq; };
	
	Shader *getShader() { return shader; };
	void setShader( Shader *s ) { shader = s; };

	void setTexCoords( const vec2& tc );
	void setNormals( const vec3& n );

	IsectType clip( const vec4& clipPlane, SpaceType space );
	IsectType clip( const vec4& clipPlane, SpaceType space, Face& newFace );
	IsectType split( const vec4& splitPlane, SpaceType space, Face& posFace, Face& negFace );

	IsectType clipToBBoxOctant( BoundingBox& bb, unsigned char octant, SpaceType space, Face& newFace );
	IsectType isectBBoxOctant( BoundingBox& bb, unsigned char octant, SpaceType space );

	IsectType bboxIsect( const BoundingBox& bb, SpaceType space );

	void cameraSpace( const Cam& camera );
	void screenSpace( const Cam& camera );

	void resize( int numPoints );

	void tessellate( List<Triangle>& newTriangles );

	friend ostream& operator<< (ostream& s, Face& t );

private:

	List<vec3> wPoints;
	List<vec3> cPoints;
	List<vec3> lPoints;
	List<vec3> sPoints;
	List<vec2> tPoints;
	List<vec3> normals;
	vec4 pEq;
	Shader *shader;
};


#endif