#include "FileTexture2D.h"
#include "Sample.h"
#include "Scene.h"
#include "stdio.h"

//
//  Attribute index definition
//
int FileTexture2D::kImageName;

//
//  Constructor just initializes attributes
//
FileTexture2D::FileTexture2D()
: imageResU(0), imageResV(0), image(NULL)
{
	FileTexture2D::initializeAttributes();
}

//
//  Attribute initialization
//
void FileTexture2D::initializeAttributes()
{
	Texture2D::initializeAttributes();

	kImageName = addAttr( new AttributeString( "imagename", CONSTANT, "" ) );
}

//
//  Evaluates the color of the texture map at the given sample position
//
vec3 FileTexture2D::vector()
{
	// get the uv value and transform it
	Sample& sample = Attribute::curSample();
	vec2 coords = fTransform * sample.texCoords2d;

	// figure out the coordinates in texture image space
	coords[VX] = fmod( coords[VX], 1.0 );
	coords[VY] = fmod( coords[VY], 1.0 );

	if( coords[VX] < 0.0 ) {
		coords[VX] = 1.0 + coords[VX];
	}

	if( coords[VY] < 0.0 ) {
		coords[VY] = 1.0 + coords[VY];
	}

	int texelU = fastDblToInt((coords[VX] * (imageResU-1)));
	int texelV = fastDblToInt((coords[VY] * (imageResV-1)));

	// return the color at that position
	return image[texelV*imageResU + texelU];
}

// 
//  Scalar value is just luminance of the texture
//
double FileTexture2D::scalar()
{
	vec3 vec = vector();
	return vec.length();
}

//
//  Reads in the texture from file.
//
bool FileTexture2D::postReadFunc( const Scene& scene )
{
	if( Texture2D::postReadFunc(scene) ) {
		if( !attrActive() ) return true;
	
		// open the texture file
		char header[8];
		FILE *imageFile = fopen( attrImageName(), "rb" );
		assert( imageFile != NULL );

		int i = 0;

		// read P6
		while( !isspace(header[i++] = fgetc(imageFile)) ) ;
		i = 0;

		// read X
		while( !isspace(header[i++] = fgetc(imageFile)) ) ;
		imageResU = atoi( header );
		i = 0;

		// read Y
		while( !isspace(header[i++] = fgetc(imageFile)) ) ;
		imageResV = atoi( header );
		i = 0;

		// read max
		while( !isspace(header[i++] = fgetc(imageFile)) ) ;
		int maxPixel = atoi( header );
		i = 0;

		// read the image data
		unsigned char *byteArray = new unsigned char[imageResU*imageResV*3];
		fread( (void *)byteArray, sizeof(unsigned char), imageResU*imageResV*3, imageFile );

		// change into vec3
		image = new vec3[imageResU*imageResV];
		for( int imageIndex = 0; imageIndex < imageResU*imageResV; imageIndex++ ) {

			int curU = imageIndex%imageResU;
			int curV = (imageResV-1)-(imageIndex/imageResU);

			int R = imageIndex*3;
			int G = R+1;
			int B = R+2;

			image[imageResU*curV+curU] = vec3( (float)byteArray[R]/(float)maxPixel,
											   (float)byteArray[G]/(float)maxPixel,
											   (float)byteArray[B]/(float)maxPixel
											   );
		}

		delete [] byteArray;
		fclose( imageFile );
		return true;
	} else {
		return false;
	}
}

