#ifndef IntListList_h
#define IntListList_h

#include "List.h"
#include <iostream.h>

/****************************************************************

	Class:		IntListList
	Purpose:	Represents a list of lists of integers.  Stored
				as a flat array for the lists along with an array
				of offsets pointing to the sublists.

				The list of lists is implemented by two lists,
				the flat array fFlatLists, and the index array
				fIndices.  fIndices is a list of (index,size)
				pairs indicating the starting index and size of
				the lists in the flat array.

*****************************************************************/

class IntListList
{
public:

	IntListList( int size = 10 );

	// the number of lists
	int numLists() const;

	// get/set the size of a particular list
	int getListSize( int whichList ) const;
	void setListSize( int whichList, int newSize );

	// get the index in the fFlatLists array of a particular list
	int getListIndex( int whichList ) const;

	// gets a pointer to a particular list
	int *getList( int whichList );

	// sets a particular list
	void setList( int whichList, int *newList, int newSize );

	// adds a new list
	int addList( int *list, int listSize );

	// frees an existing list
	void freeList( int whichList );

	// appends a new set of lists to the end of this one, 
	// relocating the new list entries by the specified amount
	void appendAndRelocate( IntListList& other, int newRoot );

	// shrinkwraps the structure so that no space is wasted
	void shrinkWrap();

	// asignment operator - copies entire structure
	IntListList& operator=( const IntListList& other );

	// read/write methods
	bool read( istream& s );
	void write( ostream& s );

private:

	List<int> fFlatLists;	// stores the actual lists in a flat array
	List<int> fIndices;		// stores indices into the flat list array
							// indicating where the lists start and how
							// big they are.

};

#endif