#include "LambertShader.h"
#include "Sample.h"
#include "Light.h"
#include "Scene.h"
#include "Texture.h"
#include "glut.h"

//
//  Attribute index definitions
//
int LambertShader::kAmbientCoeff;
int LambertShader::kAmbientColor;
int LambertShader::kDiffuseCoeff;
int LambertShader::kDiffuseColor;
int LambertShader::kTransparency;


//
//  Constructor just initializes attributes
//
LambertShader::LambertShader()
{
	LambertShader::initializeAttributes();
}

//
//  Attribute initialization
//
void LambertShader::initializeAttributes()
{
	Shader::initializeAttributes();

	kAmbientCoeff = addAttr( new AttributeDouble(	"ambientcoeff", TEXTURABLE, 0.0 ) );
	kAmbientColor = addAttr( new AttributeVec3(		"ambientcolor", TEXTURABLE, vec3(0.0) ) );
	kDiffuseCoeff = addAttr( new AttributeDouble(	"diffusecoeff", TEXTURABLE, 0.0 ) );
	kDiffuseColor = addAttr( new AttributeVec3(		"diffusecolor", TEXTURABLE, vec3(0.0) ) );
	kTransparency = addAttr( new AttributeVec3(     "transparency", TEXTURABLE, vec3(0.0) ) );
}

//
//  Compute color method, implements Lambertian shading model
//
vec3 LambertShader::computeColor( List<Light *>& ll )
{
	int numLights = ll.size();
	vec3 resultColor(0.0);
	Sample& sample = Attribute::curSample();

	vec3& sNormal = sample.worldNormal;
	vec3& sPoint = sample.worldCoords;

	// loop through the light list and accumulate contributions
	for( int i = 0; i < numLights; i++ ) {
		
		Light *curLight = ll[i];

		// get direction to light
		vec3 lPoint = curLight->worldPosition();
		vec3 L = lPoint - sPoint;
		L.normalize();

		double NdotL = sNormal * L;

		resultColor += attrAmbientCoeff() * prod( attrAmbientColor(), curLight->attrAmbient() );
		if( NdotL > 0.0 ) {
			resultColor += attrDiffuseCoeff() * NdotL * prod( attrDiffuseColor(), curLight->attrDiffuse() );

		}
	}
	return resultColor.clamp(0.0,1.0);
}

//
//  Transparency is just value of transparency attribute
//
vec3 LambertShader::computeTransparency()
{
	return attrTransparency();
}

//
//  Setup OpenGL material state
//
void LambertShader::setupOGL()
{
}

