#ifndef LambertShader_h
#define LambertShader_h

#include "Shader.h"

class Texture;

/****************************************************************

	Class:		LambertShader
	Purpose:	Implements Lambertian shading model	

*****************************************************************/
class LambertShader : public Shader
{
public:
	LambertShader();

	// attribute initialization
	virtual void initializeAttributes();

	// attribute indices
	static int kAmbientCoeff;		
	static int kAmbientColor;
	static int kDiffuseCoeff;
	static int kDiffuseColor;
	static int kTransparency;

	// attribute accessors
	double attrAmbientCoeff() { return findAttr(kAmbientCoeff)->asDouble(); };
	vec3&  attrAmbientColor() { return findAttr(kAmbientColor)->asVec3(); };
	double attrDiffuseCoeff() { return findAttr(kDiffuseCoeff)->asDouble(); };
	vec3&  attrDiffuseColor() { return findAttr(kDiffuseColor)->asVec3(); };
	vec3&  attrTransparency() { return findAttr(kTransparency)->asVec3(); };

	// color/transparency compute methods
	virtual vec3 computeColor( List<Light *>& lights );
	virtual vec3 computeTransparency();

	// setup OpenGL material state
	virtual void setupOGL();

};

#endif

