#ifndef Light_h
#define Light_h

#include "Model.h"
#include "List.h"

/****************************************************************

	Class:		Light
	Purpose:	Represents a point light with separate ambient, 
				diffuse, and specular contributions.

	File Attributes:

				ambient		<vec3> - ambient color
				diffuse		<vec3> - diffuse color
				specular	<vec3> - specular color

*****************************************************************/
class Light : public Model
{
public:
	Light();
	Light( const vec3& ambient, const vec3& diffuse, const vec3& specular );

	// attribute initialization
	virtual void initializeAttributes();

	// attribute indices
	static int kAmbient;
	static int kDiffuse;
	static int kSpecular;

	// attribute accessors
	vec3&	attrAmbient() { return findAttr(kAmbient)->asVec3(); };
	vec3&	attrDiffuse() { return findAttr(kDiffuse)->asVec3(); };
	vec3&	attrSpecular() { return findAttr(kSpecular)->asVec3(); };

	
	// world position accessor
	vec3 worldPosition() { return fWorldPos; };

	// updates transform and computes world position
	virtual void updateTransform();

protected:

	vec3 fWorldPos;		// light position in world space
};

#endif
