#ifndef Model_h
#define Model_h

#include "algebra3.h"
#include "NamedEntity.h"


/****************************************************************

	Class:		Model
	Purpose:	A model represents a 3-dimensional space.	Spaces
				can be arranged into tree hierarchies.

	File Attributes:
				translate	<vec3>		- translation
				rotate		<vec3>		- rotation about each axis
				scale		<vec3>		- scale
				parent		<string>	- name of parent model

	
				The transformation that gets applies is actually
				the scale followed by the rotate followed by the 
				translate.
				
*****************************************************************/

class Model : public NamedEntity
{
public:

	// constructors
	Model();
	Model( const char *name, const mat4& transform );



	// attribute initialization
	virtual void initializeAttributes();

	// attribute indices
	static int kTranslate;
	static int kRotate;
	static int kScale;
	static int kParent;

	// attribute accessors
	vec3&	attrTranslate() { return findAttr(kTranslate)->asVec3(); };
	vec3&	attrRotate()	{ return findAttr(kRotate)->asVec3(); };
	vec3&	attrScale()		{ return findAttr(kScale)->asVec3(); };
	char * attrParent() { return findAttr(kParent)->asString(); };

	 

	// called after file read - updates parent pointer
	virtual bool postReadFunc( const Scene& scene );

	//
	//  Transformation accessors
	//

	// get cumulative transform (local + parent)
	mat4 getCtransform() const { return cumulativeTform; };

	// get cumulative normal transform
	mat4 getCNtransform() const { return cumulativeNormTform; };
	
	// recompute cumulative tforms
	virtual void updateTransform();

	// determine whether transform is animated
	virtual bool animatedTransform();

protected:

	Model *parent;				// pointer to parent model

	mat4 localTform;			// local transform
	mat4 localNormTform;		// local normal transform
	mat4 localTformTR;			// just rotate translate
	mat4 localNormTformTR;		// just rotate translate stuff

	mat4 cumulativeTform;		// cumulative transform
	mat4 cumulativeNormTform;	// cumulative transform for normals
	mat4 cumulativeTformTR;		// just rotate translate
	mat4 cumulativeNormTformTR;	// just rotate translate

};


#endif

