#include <string.h>
#include "NamedEntity.h"
#include "algebra3.h"
#include "Attributes.h"

// 
//  Attr index definitions
//
int NamedEntity::kName;


//
//	Constructor just initializes attribute list
//
NamedEntity::NamedEntity()
{
	NamedEntity::initializeAttributes();
}

// 
//	Initialize NamedEntity attributes
//
void NamedEntity::initializeAttributes()
{
	// initialize the attribute list
	attrList = new List<Attribute *>(6);

	// add the 'name' attribute
	kName = addAttr( new AttributeString( "name", CONSTANT, "" ) );
}

//
//	Adds a new attribute to the list, returns its index
//
int NamedEntity::addAttr( Attribute *newAttr )
{
	int newIndex = attrList->size();
	attrList->append( newAttr );
	return newIndex;
}

//
//	Get a particular attribute based on its index
//
Attribute *NamedEntity::findAttr( int attrIdx )
{
	assert( attrList != NULL );
	return (*attrList)[attrIdx];
}

//
//	Get a particular attribute based on its name
//
Attribute *NamedEntity::findAttr( char *attrName )
{
	assert( attrList != NULL );
	return find<Attribute>( attrName, *attrList );
}

//
//  postReadFunc is to be called after the NamedEntity has been
//  read from a file - it calls the postReadFuncs for all the 
//  attributes in the attribute list.
//
bool NamedEntity::postReadFunc( const Scene& scene )
{
	for( int i = 0; i < attrList->size(); i++ ) {

		if( ! (*attrList)[i]->postReadFunc(scene) ) {
			return false;
		}
	}
	return true;
}

//
//  This is to be called whenever the animation timer is
//  advanced.  It flags any animated attributes as dirty.
//
void NamedEntity::setAnimDirty()
{
	for( int i = 0; i < attrList->size(); i++ ) {
		if( (*attrList)[i]->isAnimated() ) {
			(*attrList)[i]->setDirty();
		}
	}
}

//
//	Read a NamedEntity from a file
//
bool NamedEntity::read( istream& s )
{
	char c;
	int numAttrs = attrList->size();
	bool isValid = true;

	// eat the opening brace
	eatWS(s);
	s >> c;
	if( c != '{' ) return false;

	// read the attributes
	while( isValid ) {

		char temp[100];
		eatWS(s);

		s >> temp;

		if( temp[0] == '}' ) break;

		Attribute *curAttr = findAttr( temp );

		if( curAttr != NULL ) {

			if( curAttr->read(s) == ERROR ) {
				isValid = false;
			}

		} else {
			cout << "Invalid attribute : " << temp << endl;
			isValid = false;

		}
	}

	return isValid;
}

//
//	Write a NamedEntity to a file
//
void NamedEntity::write( ostream& s )
{
	s << "{" << endl;
	
	int numAttrs = attrList->size();
	for( int curAttr = 0; curAttr < numAttrs; curAttr++ ) {
		(*attrList)[curAttr]->write( s );
		s << endl;
	}

	s << "}" << endl;
}

