#ifndef NamedEntity_h
#define NamedEntity_h

#include <iostream.h>
#include "string.h"
#include "List.h"
#include "Attributes.h"

class Scene;

/****************************************************************

	Class:		NamedEntity
	Purpose:	This is the base class for all entities in the system.
				It stores a list of "attributes" which represent data
				items stored in the file format for a particular object.

				This class provides one attribute - the name of the 
				entity - that is common to all objects in the system.

	File attributes:
				name	<string>

*****************************************************************/

class NamedEntity
{
public:

	// initialize name
	NamedEntity();

	//
	//  Routines for manipulating the attributes of an entity - 
	//  reading, writing, adding, and locating attributes
	//

	// input/output routines 
	virtual bool read( istream& s );
	virtual void write( ostream& s );

	// adds a new attribute
	int addAttr( Attribute *newAttr );

	// finds a particular attribute, either by index or by name
	Attribute* findAttr( int attrId );
	Attribute* findAttr( char *attrName );



	//
	//  Routines for the attributes common to all NamedEntitys
	//

	// attribute initialization
	virtual void initializeAttributes();

	// attribute indices
	static int kName;		// (string) - the name of the entity

	// attribute accessor methods
	char *attrName()	{ return findAttr(kName)->asString(); };



	//
	//  Other useful functions for all NamedEntitys
	//

	// function gets called after file read
	virtual bool postReadFunc( const Scene& scene );

	// function to mark animated attributes as dirty
	virtual void setAnimDirty();

	// comparison operator for NamedEntitys
	int operator==( const char *rhs ) { return ( strcmp(attrName(),rhs) == 0 ); };

protected:

	List<Attribute *> *attrList;	// the attributes of the entity
};

#endif