#include "Noise3D.h"
#include "TextureTools.h"
#include <stdlib.h>

/*******************************************************************
					Static noise class stuff
********************************************************************/
#define TABMASK			(TABSIZE-1)
#define PERM(x)			perm[(x)&TABMASK]
#define INDEX(ix,iy,iz)	PERM((ix)+PERM((iy)+PERM(iz)))

#define RANDNBR  (rand()/(double)RAND_MAX)

float Noise3D::valueTab[TABSIZE];
bool Noise3D::init = false;

// random permutation table
unsigned char Noise3D::perm[TABSIZE] = {
	225, 155, 210, 108, 175, 199, 221, 144, 203, 116,  70, 213,  69, 158,  33, 252,
	  5,  82, 173, 133, 222, 139, 174,  27,   9,  71,  90, 246,  75, 130,  91, 191,
	169, 138,   2, 151, 194, 235,  81,   7,  25, 113, 228, 159, 205, 253, 134, 142,
	248,  65, 224, 217,  22, 121, 229,  63,  89, 103,  96, 104, 156,  17, 201, 129,
	 36,   8, 165, 110, 237, 117, 231,  45, 132, 211, 152,  20, 181, 111, 239, 218,
	170, 163,  51, 172, 157,  47,  80, 212, 176, 250,  87,  49,  99, 242, 136, 189,
	162, 115,  44,  43, 124,  94, 150,  16, 141, 247,  32,  10, 198, 223, 255,  72,
	 53, 131,  84,  57, 220, 197,  58,  50, 208,  11, 241,  28,   3, 192,  62, 202,
	 18, 215, 153,  24,  76,  41,  15, 179,  39,  46,  55,   6, 128, 167,  23, 188,
	106,  34, 187, 140, 164,  73, 112, 182, 244, 195, 227,  13,  35,  77, 196, 185,
	 26, 200, 226, 117,  31, 123, 168, 125, 249,  68, 183, 230, 177, 135, 160, 180,
	 12,   1, 243, 148, 102, 166,  39, 239, 251,  37, 240, 126,  64,  74, 161,  40,
	184, 149, 171, 178, 101,  66,  29,  59, 146,  61, 254, 107,  42,  86, 154,   4,
	236, 232, 120,  21, 233, 209,  45,  98, 193, 114,  78,  19, 206,  14, 118, 127,
	 48,  79, 147,  85,  30, 207, 219,  54,  88, 234, 190, 122, 95,   67, 143, 109,
	137, 214, 145,  93,  92, 100, 245,   0, 216, 186,  60,  83, 105,  97, 204,  52
};
	

// initialize the noise lattice
void Noise3D::valueTabInit( int seed )
{
	float *table = valueTab;

	srand(seed);
	for( int i = 0; i < TABSIZE; i++ ) {
		*table++ = (float)RANDNBR;
	}
}

// evaluate the noise lattice at a specified point
float Noise3D::vLattice( int ix, int iy, int iz )
{
	return valueTab[INDEX(ix,iy,iz)];
}


/********************************************************************
				Class stuff
*********************************************************************/

//
//  Constructor just initializes attributes
//
Noise3D::Noise3D()
{
	Noise3D::initializeAttributes();
}

//
//  Attribute initialization
//
void Noise3D::initializeAttributes()
{
	Texture3D::initializeAttributes();
}

//
//  After file read, initialize the noise table
//
bool Noise3D::postReadFunc( const Scene& scene ) 
{
	if( Texture3D::postReadFunc(scene) ) {
		if( !init ) {
			valueTabInit(665);
			init = true;
		}
		return true;
	} else {
		return false;
	}
}

//
//  Scalar evaluation.  Tricubic interpolation of lattice values
//
double Noise3D::scalar()
{
	int ix, iy, iz;
	double fx, fy, fz;
	double xknots[4], yknots[4], zknots[4];
	
	Sample& curSample = Attribute::curSample();
	vec3 coords = fTransform * curSample.texCoords3d;

	ix = fastDblToInt(floor(coords[VX]));
	fx = coords[VX] - ix;

	iy = fastDblToInt(floor(coords[VY]));
	fy = coords[VY] - iy;

	iz = fastDblToInt(floor(coords[VZ]));
	fz = coords[VZ] - iz;

	for( int k = -1; k <= 2; k++ ) {
		for( int j = -1; j <= 2; j++ ) {
			for( int i = -1; i <= 2; i++ ) {
				xknots[i+1] = vLattice(ix+i, iy+j, iz+k);
			}
			yknots[j+1] = spline<double>(fx, 4, xknots);
		}
		zknots[k+1] = spline<double>(fy, 4, yknots);
	}

	double ret = spline<double>(fz,4,zknots);
	return ret;
}

//
//  Vector just returns 3 scalars.
//
vec3 Noise3D::vector()
{
	double val = scalar();

	return vec3(val);
}
