#include "Sphere.h"
#include "Face.h"

const double X = 0.525731112119133606;
const double Z = 0.850650808352039932;

/*
static double vdata[12][3] = {
	{-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},
	{0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},
	{Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0}
};

static int tindices[20][3] = {
	{0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},
	{8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},
	{7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6},
	{6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11},
	
};
*/



static int tindices[24][3] = {
	{0,4,1},  {0,9,4}, {4,8,1},
	{8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},
	{7,10,3}, {11,0,6}, {0,1,6},
	{6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11},

	{9,5,12}, {12,4,9}, {8,4,12}, {12,5,8},
	{7,13,10}, {13,6,10}, {7,11,13}, {13,11,6}
	
};

static double vdata[14][3] = {
	{-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},
	{0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},
	{Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0},
	{0.0, 1.0, 0.0}, {0.0, -1.0, 0.0}
};





Sphere::Sphere() : tessLevel(0) 
{
}


void Sphere::tessellateRec( List<Face>& tessellation, vec3& v1, vec3& v2, vec3& v3, int level )
{
	vec3 v12, v23, v31;
	bool split = false;
	Face posFace(4), negFace(4);

	if( level == 0 ) {

		Face newFace;

		newFace.resize(3);

		newFace.local(0) = v1;
		newFace.normal(0) = v1;
		newFace.texture(0) = computeAngles( v1 );

		newFace.local(1) = v2;
		newFace.normal(1) = v2;
		newFace.texture(1) = computeAngles( v2 );

		newFace.local(2) = v3;
		newFace.normal(2) = v3;
		newFace.texture(2) = computeAngles( v3 );

		
		if( (newFace.local(0)[VX] >= 0.0) && (newFace.local(1)[VX] >= 0.0) && (newFace.local(2)[VX] >= 0.0) ) {
			double z1 = newFace.local(0)[VZ];
			double z2 = newFace.local(1)[VZ];
			double z3 = newFace.local(2)[VZ];

			if( ((z1>0.0)||(z2>0.0)||(z3>0.0)) && ((z1<0.0)||(z2<0.0)||(z3<0.0)) ) {
				split = true;
				vec4 splitPlane(0.0,0.0,1.0,0.0);

				posFace.resize(0);
				negFace.resize(0);
				newFace.split( splitPlane, LOCAL, posFace, negFace );

				for( int i = 0; i < posFace.size(); i++ ) {
					posFace.texture(i) = computeAngles( posFace.local(i) );
				}

				for( i = 0; i < negFace.size(); i++ ) {
					negFace.texture(i) = computeAngles( negFace.local(i) );
					if( fabs(negFace.local(i)[VZ]) < 0.000001 ) {
						negFace.texture(i)[VX] = 1.0;
					}
				}

			} else if( (z1<=0.0)&&(z2<=0.0)&&(z3<=0.0) ) {
				if( fabs(z1) <= 0.000001 ) {
					newFace.texture(0)[VX] = 1.0;
				}
				if( fabs(z2) <= 0.000001 ) {
					newFace.texture(1)[VX] = 1.0;
				}
				if( fabs(z3) <= 0.000001 ) {
					newFace.texture(2)[VX] = 1.0;
				}
			}
		}

		split = false;

		if( split ) {
			posFace.setShader( shader );
			negFace.setShader( shader );

			for( int i = 0; i < posFace.size(); i++ ) {
				vec3& norm = posFace.normal(i);
				posFace.world(i) = cumulativeTform * posFace.local(i);
				norm = vec3( cumulativeNormTform * vec4( norm, 0.0 ), VW );
				posFace.normal(i).normalize();

				if( fabs( posFace.local(i)[VY]-1.0 ) <= 0.000001 ) {
					assert( posFace.size() == 3 );
					posFace.texture(i)[VX] = 0.5*(posFace.texture((i+1)%3)[VX] + posFace.texture((i+2)%3)[VX]);
				}
			}

			for( i = 0; i < negFace.size(); i++ ) {
				vec3& norm = negFace.normal(i);
				negFace.world(i) = cumulativeTform * negFace.local(i);
				norm = vec3( cumulativeNormTform * vec4( norm, 0.0 ), VW );
				negFace.normal(i).normalize();

				if( fabs( negFace.local(i)[VY]-1.0 ) <= 0.000001 ) {
					assert( negFace.size() == 3 );
					negFace.texture(i)[VX] = 0.5*(negFace.texture((i+1)%3)[VX] + negFace.texture((i+2)%3)[VX]);
				}

			
			}

			posFace.plane() = computeTriPlaneEq( posFace.world(0), posFace.world(1), posFace.world(2) );
			negFace.plane() = computeTriPlaneEq( negFace.world(0), negFace.world(1), negFace.world(2) );

			//cout << "Texture split positive: " << posFace.plane() << endl;
			//cout << "Texture split negative: " << negFace.plane() << endl;

			tessellation.append( posFace );
			tessellation.append( negFace );

		} else {
			newFace.setShader( shader );
	
			for( int j = 0; j < 3; j++ ) {
				vec3& norm = newFace.normal(j);
				newFace.world(j) = cumulativeTform * newFace.local(j);
				norm = vec3( cumulativeNormTform * vec4( norm, 0.0 ), VW );
				newFace.normal(j).normalize();
			} 

			if( fabs(newFace.local(0)[VY]-1.0) <= 0.000001 ) {
				newFace.texture(0)[VX] = 0.5*(newFace.texture(1)[VX] + newFace.texture(2)[VX]);
			} 

			newFace.plane() = computeTriPlaneEq( newFace.world(0), newFace.world(1), newFace.world(2) );
			tessellation.append(newFace);

			//cout << "Normal: " << newFace.plane() << endl;
		}

		return;
	}

	v12 = (v1 + v2).normalize();
	v23 = (v2 + v3).normalize();
	v31 = (v3 + v1).normalize();

	tessellateRec( tessellation, v1, v12, v31, level-1 );
	tessellateRec( tessellation, v2, v23, v12, level-1 );
	tessellateRec( tessellation, v3, v31, v23, level-1 );
	tessellateRec( tessellation, v12, v23, v31, level-1 );
}

	

void Sphere::tessellate( List<Face>& tessellation )
{
	for( int i = 0; i < 24; i++ ) {
		int i1 = tindices[i][2];
		int i2 = tindices[i][1];
		int i3 = tindices[i][0];

		vec3 v1( vdata[i1][0], vdata[i1][1], vdata[i1][2] );
		vec3 v2( vdata[i2][0], vdata[i2][1], vdata[i2][2] );
		vec3 v3( vdata[i3][0], vdata[i3][1], vdata[i3][2] );

		tessellateRec( tessellation, v1, v2, v3, tessLevel );
	}
}

int Sphere::faceCount()
{
	return 20 * (int)pow( 4, tessLevel );
}

bool Sphere::read( istream& s )
{
	enum { TESSLEVEL=1,
		   CLOSEBR=2,
		   UNKNOWN=3
	};

	static char *key_words[] = { "TESSLEVEL",
								 "}",
							     0
	};

	int key;

	if( !Shape::read(s) ) {
		return false;
	}

	key = parseInput( s, key_words );
	if( key == TESSLEVEL ) {
		s >> tessLevel;
	} else {
		return false;
	}

	key = parseInput( s, key_words );
	if( key != CLOSEBR ) {
		return false;
	}

	return true;
}

void Sphere::write( ostream& s )
{
	Shape::write(s);
	s << endl;

	s << "******Sphere {" << endl;
	s << "Tessellation level: " << tessLevel << endl;

	s << "******Sphere }";
}