#include "PhongShader.h"
#include "Sample.h"
#include "Light.h"
#include "Texture.h"
#include "Scene.h"
#include "glut.h"

//
//  Attribute index definitions
//
int PhongShader::kSpecularCoeff;
int PhongShader::kSpecularColor;
int PhongShader::kSpecularExp;

//
//  Constructor just initializes attributes
//
PhongShader::PhongShader()
{
	PhongShader::initializeAttributes();
}

//
//  Attribute initialization
//
void PhongShader::initializeAttributes()
{
	LambertShader::initializeAttributes();

	kSpecularCoeff = addAttr( new AttributeDouble( "specularcoeff", TEXTURABLE, 0.0 ) );
	kSpecularColor = addAttr( new AttributeVec3( "specularcolor", TEXTURABLE, vec3(0.0) ) );
	kSpecularExp = addAttr( new AttributeDouble( "specularexp", TEXTURABLE, 0.0 ) );
}

//
//	Implements the Phong shading model
//
vec3 PhongShader::computeColor( List<Light *>& ll )
{
	vec3 diffuseResult;
	vec3 resultColor(0.0);
	vec3 N, V, R;
	Sample& sample = Attribute::curSample();

	vec3 sPoint = sample.worldCoords; 
	int numLights = ll.size();

	N = sample.worldNormal.normalize();
	V = (sample.eye - sample.worldCoords).normalize();
	
	resultColor = LambertShader::computeColor( ll );

	for( int i = 0; i < numLights; i++ ) {
		Light *curLight = ll[i];

		vec3 L = (curLight->worldPosition() - sPoint).normalize();
		vec3 R = ((2.0 * (N*L))*N - L).normalize();

		double RdotV = R * V;
		double NdotL = N * L;

		if( (NdotL > 0.0) && (RdotV > 0.0) ) {
			resultColor += attrSpecularCoeff() * pow( RdotV, attrSpecularExp()) * 
						   prod( attrSpecularColor(), curLight->attrSpecular() );
		}
	}

	return resultColor.clamp(0.0,1.0);
}

void PhongShader::setupOGL()
{
}

