#ifndef Poly_h
#define Poly_h

#include "Shape.h"

/****************************************************************

	Class:		Poly
	Purpose:	Represents a polygon mesh, that can optionally have
				normals and/or texture coordinates at its vertices.

	File Attributes:

				objpoints	<vec3 list>	 - mesh vertices in object space
				texpoint	<vec2 list>  - texture coordinates of mesh
										   vertices
				normals		<vec3 list>  - normals at mesh vertices
				facelist	<int list list> - list of faces

*****************************************************************/
class Poly : public Shape
{
public:
	Poly();

	// attribute initialization
	virtual void initializeAttributes();

	// attribute indices
	static int kObjPoints;
	static int kTexPoints;
	static int kNormals;
	static int kFaceList;

	// attribute accessors
	List<vec3>&		attrObjPoints() { return findAttr(kObjPoints)->asVec3List(); };
	List<vec2>&		attrTexPoints() { return findAttr(kTexPoints)->asVec2List(); };
	List<vec3>&		attrNormals()	{ return findAttr(kNormals)->asVec3List();	 };
	IntListList&	attrFaceList()	{ return findAttr(kFaceList)->asIntListList(); };

	// post-read function
	virtual bool postReadFunc( const Scene& scene );

	// local tessellation method
	void localTessellate( TessOptions options, SpaceType space );

	// counts faces
	int faceCount();

	// computes bounding box
	void computeBB( const mat4& transform );

	// retrieves tessellation options
	virtual TessOptions getTessOptions();

protected:

	bool		fHasNormals;		// does the poly have normals?
	bool		fHasTexCoords;		// does the poly have texture coords?

};

#endif