#ifndef RefMesh_h
#define RefMesh_h

#include "Poly.h"

/****************************************************************

	Class:		RefMesh
	Purpose:	Poly mesh read in from Wavefront OBJ file format.
	
	File Attributes:
				
				filename	<string> - name of the OBJ file
				facecount	<int>	 - estimate of number of faces 
									   in the file

*****************************************************************/
class RefMesh : public Shape
{
public:

	RefMesh();

	// attribute initialization
	void initializeAttributes();

	// attribute indices
	static int kFilename;
	static int kFaceCount;
	static int kTriangulate;


	// attribute accessors
	char *& attrFilename() { return findAttr(kFilename)->asString(); };
	int&	attrFaceCount() { return findAttr(kFaceCount)->asInt(); };
	bool	attrTriangulate() { return findAttr(kTriangulate)->asBool(); };
	
	// post read func actually reads the mesh data from the file
	virtual bool postReadFunc( const Scene& scene );

	// local tessellation method
	void localTessellate( TessOptions options, SpaceType space );

	// counts faces
	int faceCount();

	// computes bounding box
	void computeBB( const mat4& transform );

};

#endif