#include <string.h>
#include <fstream.h>
#include "Scene.h"
#include "Shape.h"
#include "Cam.h"
#include "CamZB.h"
#include "CamRT.h"
#include "CamHCM.h"
#include "CamOpenGL.h"
#include "Poly.h"
#include "LambertShader.h"
#include "PhongShader.h"
#include "Light.h"
#include "FlatShader.h"
#include "Sphere.h"
#include "Cube.h"
#include "Checker2D.h"
#include "Checker3D.h"
#include "Noise3D.h"
#include "FileTexture2D.h"
#include "RefMesh.h"
#include "Tessellation.h"
#include "AnimCurve.h"

//
//  Reads the scene from a file, then calls the postReadFunc for each
//  object that was read.
//
bool Scene::readScene( istream& s )
{
	enum {	ZBCAMERA=1,
			HCMCAMERA=2,
			RTCAMERA=3,
			OPENGLCAMERA=4,

			POLY=5,
			SPHERE=6,
			CUBE=7,
			REFERENCE=8,

			FLATSHADER=9,
			LAMBERTSHADER=10,
			PHONGSHADER=11,

			LIGHT=12,

			CHECKER2D=13,
			CHECKER3D=14,
			FILETEXTURE2D=15,
			NOISE3D=16,

			ANIMCURVESCALAR=17,
			ANIMCURVEVEC2=18,
			ANIMCURVEVEC3=19,

			UNKNOWN=20
	};

	static char *key_words[] = {	"ZBCAMERA",
									"HCMCAMERA",
									"RTCAMERA",
									"OPENGLCAMERA",

									"POLY",
									"SPHERE",
									"CUBE",
									"REFERENCE",

									"FLATSHADER",
									"LAMBERTSHADER",
									"PHONGSHADER",

									"LIGHT",

									"CHECKER2D",
									"CHECKER3D",
									"FILETEXTURE2D",
									"NOISE3D",

									"ANIMCURVESCALAR",
									"ANIMCURVEVEC2",
									"ANIMCURVEVEC3",

									NULL
	};

	int key;
	CamZB			*camZB			= NULL;
	CamRT			*camRT			= NULL;
	CamHCM			*camHCM			= NULL;
	LambertShader	*lambertShader	= NULL;
	PhongShader		*phongShader	= NULL;
	FlatShader		*flatShader		= NULL;
	Poly			*poly			= NULL;
	Light			*light			= NULL;
	Sphere			*sphere			= NULL;
	Cube			*cube			= NULL;
	Checker2D		*checker2d		= NULL;
	Checker3D		*checker3d		= NULL;
	CamOpenGL		*camOpenGL		= NULL;
	FileTexture2D	*fileTexture2d	= NULL;
	RefMesh			*refMesh		= NULL;
	Noise3D			*noise3d		= NULL;
	AnimCurveScalar *acScalar		= NULL;
	AnimCurveVec2	*acVec2			= NULL;
	AnimCurveVec3	*acVec3			= NULL;

	while( (key = parseInput(s, key_words)) && !s.eof() ) {

		switch( key ) {

			case ZBCAMERA:

				camZB = new CamZB;

				if ( !(camZB->read(s)) ) {
					return false;
				} else {
					cameraList.append( camZB );
				}

				break;

			case RTCAMERA:

				camRT = new CamRT;

				if ( !(camRT->read(s)) ) {
					return false;
				} else {
					cameraList.append( camRT );
				}

				break;

			case HCMCAMERA:

				camHCM = new CamHCM;

				if( !(camHCM->read(s)) ) {
					return false;
				} else {
					cameraList.append( camHCM );
				}
				break;

			case OPENGLCAMERA:

				camOpenGL = new CamOpenGL;

				if( !(camOpenGL->read(s)) ) {
					return false;
				} else {
					cameraList.append( camOpenGL );
				}
				break;

			case LAMBERTSHADER:

				lambertShader = new LambertShader;

				if( !(lambertShader->read(s)) ) {
					return false;
				} else {
					shaderList.append( lambertShader );
				}

				break;

			case PHONGSHADER:
	
				phongShader = new PhongShader;

				if( !(phongShader->read(s)) ) {
					return false;
				} else {
					shaderList.append( phongShader );
				}

				break;

			case FLATSHADER:

				flatShader = new FlatShader;

				if( !(flatShader->read(s)) ) {
					return false;
				} else {
					shaderList.append( flatShader );
				}
				break;

			case POLY:

				poly = new Poly;

				if( !(poly->read(s)) ) {
					return false;
				} else {
					objList.append( poly );
				}
				break;

			case CUBE:

				cube = new Cube;

				if( !(cube->read(s)) ) {
					return false;
				} else {
					objList.append( cube );
				}
				break;

			case SPHERE:

				sphere = new Sphere;

				if( !(sphere->read(s)) ) {
					return false;
				} else {
					objList.append(sphere);
				}
				break;

			case LIGHT:

				light = new Light;

				if( !(light->read(s)) ) {
					return false;
				} else {
					lightList.append( light );
				}
				break;

			case CHECKER2D:

				checker2d = new Checker2D;

				if( !(checker2d->read(s)) ) {
					return false;
				} else {
					textureList.append( checker2d );
				}
				break;

			case CHECKER3D:

				checker3d = new Checker3D;

				if( !(checker3d->read(s)) ) {
					return false;
				} else {
					textureList.append( checker3d );
				}
				break;

			case FILETEXTURE2D:

				fileTexture2d = new FileTexture2D;
				
				if( !(fileTexture2d->read(s)) ) {
					return false;
				} else {
					textureList.append( fileTexture2d );
				}
				break;

			case REFERENCE:
			
				refMesh = new RefMesh;

				if( !(refMesh->read(s)) ) {
					return false;
				} else {
					objList.append(refMesh);
				}
				break;


			case NOISE3D:

				noise3d = new Noise3D;

				if( !(noise3d->read(s)) ) {
					return false;
				} else {
					textureList.append( noise3d );
				}
				break;

			case ANIMCURVESCALAR:

				acScalar = new AnimCurveScalar;

				if( !(acScalar->read(s)) ) {
					return false;
				} else {
					animCurveList.append( acScalar );
				}
				break;

			case ANIMCURVEVEC2:

				acVec2 = new AnimCurveVec2;

				if( !(acVec2->read(s)) ) {
					return false;
				} else {
					animCurveList.append( acVec2 );
				}
				break;

			case ANIMCURVEVEC3:

				acVec3 = new AnimCurveVec3;

				if( !(acVec3->read(s)) ) {
					return false;
				} else {
					animCurveList.append( acVec3 );
				}
				break;

			case UNKNOWN:
				return false;
				break;

		}
	}
	
	// go through all entities and call their postRead functions
	for( int i = 0; i < cameraList.size(); i++ ) {
		if( !cameraList[i]->postReadFunc(*this) ) {
			return false;
		}
	}

	for( i = 0; i < lightList.size(); i++ ) {
		if( !lightList[i]->postReadFunc(*this) ) {
			return false;
		}
	}

	for( i = 0; i < objList.size(); i++ ) {
		if( !objList[i]->postReadFunc(*this) ) {
			return false;
		}
	}

	for( i = 0; i < shaderList.size(); i++ ) {
		if( !shaderList[i]->postReadFunc(*this) ) {
			return false;
		}
	}

	for( i = 0; i < textureList.size(); i++ ) {
		if( !textureList[i]->postReadFunc(*this) ) {
			return false;
		}
	}

	return true;
			
}

//
//  Writes the scene.
//
void Scene::write( ostream& s )
{

	s << "*******************" << endl;
	s << "     CAMERAS       " << endl;
	s << "*******************" << endl << endl;
	for( int i = 0; i < cameraList.size(); i++ ) {
		cameraList[i]->write(s);
		s << endl;
	}

	s << endl << endl;
	s << "*******************" << endl;
	s << "      LIGHTS       " << endl;
	s << "*******************" << endl << endl;
	for( i = 0; i < lightList.size(); i++ ) {
		lightList[i]->write(s);
		s << endl;
	}

	s << endl << endl;
	s << "*******************" << endl;
	s << "      OBJECTS      " << endl;
	s << "*******************" << endl << endl;
	for( i = 0; i < objList.size(); i++ ) {
		objList[i]->write(s);
		s << endl;
	}

	s << endl << endl;
	s << "*******************" << endl;
	s << "      SHADERS      " << endl;
	s << "*******************" << endl << endl;
	for( i = 0; i < shaderList.size(); i++ ) {
		shaderList[i]->write(s);
		s << endl;
	}

	s << endl << endl;
	s << "*******************" << endl;
	s << "     ANIMCURVES    " << endl;
	s << "*******************" << endl << endl;
	for( i = 0; i < animCurveList.size(); i++ ) {
		animCurveList[i]->write(s);
		s << endl;
	}

}


//
//  Main rendering method for the scene.  Calls the scene preProcess
//  function, then for each renderable camera, calls its preProcess
//  function, then its render() function, then its postProcess()
//  function.  Finally, the scene postProcess() function is called.
//
void Scene::render()
{
	// perform any global preprocessing 
	cout << endl << "Scene Preprocessing..." << endl << endl;
	preProcess();

	// for each camera, preProcess, then render, then postProcess
	for( int i = 0; i < cameraList.size(); i++ ) {

		Cam*& curCamera = cameraList[i];
		char *camName = curCamera->attrName();

		if( curCamera->attrRenderable() ) {

			cout << endl << "***********Camera " << camName << " preprocessing..." << endl << endl;

			// preprocess for this camera 
			curCamera->preProcess( *this );

			cout << endl << "***********Camera " << camName << " rendering..." << endl << endl;

			// render with this camera 
			curCamera->render( *this );

			cout << endl << "***********Camera " << camName << " postprocessing..." << endl << endl;

			// postprocess for this camera 
			curCamera->postProcess( *this );
		
		}
	}

	// perform any global postprocessing 
	cout << endl << "Scene postprocessing..." << endl << endl;
	postProcess();

}

//
//  Calls the localTessellate method for all non-hidden geometry
//
void Scene::localTessellate( TessOptions options, SpaceType space )
{
	for( int i = 0; i < objList.size(); i++ ) {
		if( !(objList[i]->attrHidden()) ) {
			objList[i]->localTessellate( options, space );
		}
	}
}

//
//  Constructs the scene bounding box from all non-hidden geometry
//
void Scene::buildBB()
{
	for( int i = 0; i < objList.size(); i++ ) {
		if( !(objList[i]->attrHidden()) ) {
			objList[i]->computeBB( objList[i]->getCtransform() );
			sceneBB.expand( objList[i]->getBB() );
		}
	}
}

//
//  Finds the maximum number of vertices in any local tessellation 
//  in the scene.
//
int Scene::maxVertices()
{
	int max = -1;

	for( int i = 0; i < objList.size(); i++ ) {
		Tessellation *tess = objList[i]->localTessellation();
		if( tess ) {
			int numVert = tess->vList.size();
			max = (numVert > max) ? numVert : max;
		}
	}

	return max;
}

//
//  Counts the number of faces in the scene
//
int Scene::faceCount()
{
	int count = 0;

	for( int i = 0; i < objList.size(); i++ ) {
		if( !objList[i]->attrHidden() ) {
			count += objList[i]->faceCount();
		}
	}

	return count;
}


//
//  Sets the current animation time for the scene.  Labels
//  any animated attributes as dirty.
//
void Scene::setCurTime( double newTime )
{
	double curTime = Attribute::curTime();

	if( curTime != newTime ) {

		Attribute::setCurTime(newTime);

		for( int i = 0; i < cameraList.size(); i++ ) {
			cameraList[i]->setAnimDirty();
		}

		for( i = 0; i < lightList.size(); i++ ) {
			lightList[i]->setAnimDirty();
		}

		for( i = 0; i < objList.size(); i++ ) {
			objList[i]->setAnimDirty();
		}

		for( i = 0; i < shaderList.size(); i++ ) {
			shaderList[i]->setAnimDirty();
		}

		for( i = 0; i < textureList.size(); i++ ) {
			textureList[i]->setAnimDirty();
		}
	}
}

//
//  Update the transforms of all textures at the current animation time
//
void Scene::updateTextures()
{
	for( int i = 0; i < textureList.size(); i++ ) {
		textureList[i]->updateTransform();
	}
}

//
//  Update the transform of all lights at the current animation time
//
void Scene::updateLights()
{
	for( int i = 0; i < lightList.size(); i++ ) {
		lightList[i]->updateTransform();
	}
}

//
//  Update the transform of all geometry at the current animation time
//
void Scene::updateGeometry()
{
	for( int i = 0; i < objList.size(); i++ ) {
		objList[i]->updateTransform();
	}
}




