#ifndef Shader_h
#define Shader_h

#include "algebra3.h"
#include "List.h"
#include "NamedEntity.h"

class Light;

/****************************************************************

	Class:		Shader
	Purpose:	Surface shader base class.  Takes a sample and can
				compute a color and transparency value.

*****************************************************************/

class Shader : public NamedEntity
{
public:
	Shader();
	
	// attribute initialization
	virtual void initializeAttributes();

	// color/transparency compute methods
	virtual vec3 computeColor( List<Light *>& ll ) = 0;
	virtual vec3 computeTransparency() = 0;

	// called before this shader is used in an OpenGL rendering
	virtual void preProcessOGL()	{};

	// switch the OpenGL material state to that of this shader
	virtual void setupOGL()			{};

	// called after this shader is used in an OpenGL rendering
	virtual void postProcessOGL()	{};

};

#endif
