#include "Shape.h"
#include "Scene.h"
#include "Tessellation.h"
#include <string.h>

//
//  Attribute index definitions
//
int Shape::kShaderName;
int Shape::kHidden;
int Shape::kHasNormals;
int Shape::kHasTex;

//
//  Constructor just initializes attributes
//
Shape::Shape() : fShader(NULL), localTess(NULL)
{
	Shape::initializeAttributes();
}

//
//  Attribute initialization
//
void Shape::initializeAttributes()
{
	Model::initializeAttributes();

	kShaderName = addAttr( new AttributeString( "shader", CONSTANT, "" ) );
	kHidden =	  addAttr( new AttributeBool( "hidden", CONSTANT, false ) );
	kHasNormals = addAttr( new AttributeBool( "hasnormals", CONSTANT, true ) );
	kHasTex =     addAttr( new AttributeBool( "hastex", CONSTANT, true ) );
}

//
//  Post-read function resolves surface shader, and checks for animated
//  geometry
//
bool Shape::postReadFunc( const Scene& scene ) 
{
	if( Model::postReadFunc( scene ) ) {

		// check for animated transformation matrix
		fGeometryAnimated = animatedTransform();
		
		fShader = scene.findShader( attrShaderName() );
		return (fShader != NULL) ? true : false;
	} else {
		return false;
	}
}

//
//  Retrieves the tessellation options specifies in the scene file
//  for this object.
//
TessOptions Shape::getTessOptions()
{
	TessOptions ret = kTessNone;

	if( attrHasNormals() ) {
		ret |= kTessNormals;
	}

	if( attrHasTex() ) {
		ret |= kTessTex;
	}

	return ret;
}

//
//  Frees the local tessellation
//
void Shape::deleteLocalTessellation()
{
	if( localTess ) {
		delete localTess;
		localTess = NULL;
	}
}

