#include "Sphere.h"
#include "Face.h"
#include "glut.h"
#include "Tessellation.h"

const double X = 0.525731112119133606;
const double Z = 0.850650808352039932;

int Sphere::kTessLevel;

/*
static int tindices[24][3] = {
	{0,4,1},  {0,9,4}, {4,8,1},
	{8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},
	{7,10,3}, {11,0,6}, {0,1,6},
	{6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11},

	{9,5,12}, {12,4,9}, {8,4,12}, {12,5,8},
	{7,13,10}, {13,6,10}, {7,11,13}, {13,11,6}
	
};

static double vdata[14][3] = {
	{-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},
	{0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},
	{Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0},
	{0.0, 1.0, 0.0}, {0.0, -1.0, 0.0}
};
*/

static int tindices[20][3] = {
	{0,4,1},  {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},
	{8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},
	{7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6},
	{6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11}
};

static double vdata[12][3] = {
	{-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},
	{0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},
	{Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0},
};

enum { V1=0,V2,V3,V12,V23,V31 };

static int NEWTRIS[4][3] = {
	{V1,V12,V31}, {V2,V23,V12}, {V3,V31,V23}, {V12,V23,V31}
};





Sphere::Sphere()
{
	Sphere::initializeAttributes();
}

void Sphere::initializeAttributes()
{
	Shape::initializeAttributes();

	kTessLevel = addAttr( new AttributeInt( "tesslevel", CONSTANT, 2 ) );
}

void Sphere::localTessellate( TessOptions options, SpaceType space )
{
	TessOptions allOptions = options | getTessOptions();
	int lvl = attrTessLevel();

	// count the final number of faces and vertices that there will be
	int numFaces = 20;
	int numVertices = 12;
	for( int i = 0; i < lvl; i++ ) {
		numVertices += (3*numFaces)/2;
		numFaces *= 4;
	}

	localTess = new Tessellation( allOptions, numFaces, numVertices );
	localTess->sList.append( this );

	for( i = 0; i < 12; i++ ) {
		vec3 curPoint( vdata[i][0], vdata[i][1], vdata[i][2] );
		localTess->addPoint( curPoint, curPoint, computeAngles(curPoint) );
	}

	for( i = 0; i < 20; i++ ) {
		tessellateRec( *localTess, tindices[i], lvl );
	}

	if( space == WORLD ) {
		localTess->transformVertices( getCtransform(), NULL );
		localTess->transformNormals( getCNtransform(), NULL );
	}

	if( allOptions & kTessPlaneEqs ) {
		localTess->computePlaneEqs(NULL,NULL);
	}
}

void Sphere::tessellateRec( Tessellation& tessellation, int *vList, int level )
{
	vec4 dummyPlane(0.0);

	if( level == 0 ) {
		tessellation.addFace( 3, vList, 0, NULL );
	} else {

		int newV[12];
		int vIndices[6];

		vec3 v[6];
		v[V1] = tessellation.vList[vList[0]];
		v[V2] = tessellation.vList[vList[1]];
		v[V3] = tessellation.vList[vList[2]];
		v[V12] = (v[V1] + v[V2]).normalize();
		v[V23] = (v[V2] + v[V3]).normalize();
		v[V31] = (v[V3] + v[V1]).normalize();

		for( int i = V1; i <= V3; i++ ) {
			vIndices[i] = vList[i];
		}

		for( i = V12; i <= V31; i++ ) {
			vIndices[i] = tessellation.addPoint(v[i], v[i], computeAngles(v[i]) );
		}

		for( i = 0; i < 4; i++ ) {
			for( int j = 0; j < 3; j++ ) {
				newV[3*i+j] = vIndices[NEWTRIS[i][j]];
			}
		}

		for( i = 0; i < 4; i++ ) {
			tessellateRec( tessellation, &(newV[3*i]), level-1 );
		}
	}
}


void Sphere::tessellate( Tessellation& tessellation )
{
	/*
	int vIndices[3], nIndices[3], tIndices[3];

	int firstV = tessellation.vList.size();
	int firstN = tessellation.nList->size();
	int firstT = tessellation.tList->size();

	tessellation.startObject( this, true, true );
	
	for( int i = 0; i < 12; i++ ) {
		vec3 curPoint( vdata[i][0], vdata[i][1], vdata[i][2] );
		tessellation.addVertex( curPoint );
		tessellation.addNormal( curPoint );
		tessellation.addTex( computeAngles(curPoint) );
	}

	for( i = 0; i < 20; i++ ) {

		for( int j = 0; j < 3; j++ ) {
			vIndices[j] = tindices[i][2-j] + firstV;
			nIndices[j] = tindices[i][2-j] + firstN;
			tIndices[j] = tindices[i][2-j] + firstT;
		}

		tessellateRec( tessellation, vIndices, nIndices, tIndices, attrTessLevel() );
	}

	tessellation.endObject();
	*/
}


/*

void Sphere::tessellateRec( Tessellation& tessellation, int *vList, int *nList, int *tList, int level )
						    int v1index, int v2index, int v3index, 
							int level )
{
	vec4 dummyPlane(0.0);

	if( level == 0 ) {

		int indices[3] = { v1index, v2index, v3index };
		tessellation.addFace( indices, 3, (Shape *)this, true, &dummyPlane );

	} else {

		vec3 v1( tessellation.vList[v1index] );
		vec3 v2( tessellation.vList[v2index] );
		vec3 v3( tessellation.vList[v3index] );

		vec3 v12 = (v1 + v2).normalize();
		vec3 v23 = (v2 + v3).normalize();
		vec3 v31 = (v3 + v1).normalize();

		int v12index = tessellation.vList.size();
		int v23index = v12index + 1;
		int v31index = v23index + 1;

		tessellation.addPoint( v12, v12, computeAngles( v12 ) );
		tessellation.addPoint( v23, v23, computeAngles( v23 ) );
		tessellation.addPoint( v31, v31, computeAngles( v31 ) );

		tessellateRec( tessellation, v1index, v12index, v31index, level-1 );
		tessellateRec( tessellation, v2index, v23index, v12index, level-1 );
		tessellateRec( tessellation, v3index, v31index, v23index, level-1 );
		tessellateRec( tessellation, v12index, v23index, v31index, level-1 );

	}
}

void Sphere::tessellate( Tessellation& tessellation )
{
	tessellation.startObject( this, true );
	
	int firstVertex = tessellation.vList.size();

	for( int i = 0; i < 12; i++ ) {
		vec3 curPoint( vdata[i][0], vdata[i][1], vdata[i][2] );
		tessellation.addPoint( curPoint, curPoint, computeAngles( curPoint ) );
	}

	for( i = 0; i < 20; i++ ) {
		int i1 = tindices[i][2] + firstVertex;
		int i2 = tindices[i][1] + firstVertex;
		int i3 = tindices[i][0] + firstVertex;

		tessellateRec( tessellation, i1, i2, i3, attrTessLevel() );
	}

	tessellation.endObject();
}
*/

void Sphere::computeBB( const mat4& transform )
{
	fBoundingBox = BoundingBox( vec3(-1.0), vec3(1.0) );
	fBoundingBox.applyTransform( transform );
}

/*
void Sphere::tessellateRec( List<Face>& tessellation, vec3& v1, vec3& v2, vec3& v3, int level )
{
	vec3 v12, v23, v31;
	bool split = false;
	Face posFace(4), negFace(4);

	if( level == 0 ) {

		Face newFace;

		newFace.resize(3);

		newFace.local(0) = v1;
		newFace.normal(0) = v1;
		newFace.texture(0) = computeAngles( v1 );

		newFace.local(1) = v2;
		newFace.normal(1) = v2;
		newFace.texture(1) = computeAngles( v2 );

		newFace.local(2) = v3;
		newFace.normal(2) = v3;
		newFace.texture(2) = computeAngles( v3 );

		if( (v1[VX]>=0.0) && (v2[VX]>=0.0) && (v3[VX]>=0.0) &&
			(v1[VZ]<=0.0) && (v2[VZ]<=0.0) && (v3[VZ]<=0.0)	) {

			if( fabs(v1[VZ]) <= 0.000001 ) {
				newFace.texture(0)[VX] = 1.0;
			}
			if( fabs(v2[VZ]) <= 0.000001 ) {
				newFace.texture(1)[VX] = 1.0;
			}
			if( fabs(v3[VZ]) <= 0.000001 ) {
				newFace.texture(2)[VX] = 1.0;
			}
		}

		newFace.setShader( shader );
	
		for( int j = 0; j < 3; j++ ) {
			vec3& norm = newFace.normal(j);
			newFace.world(j) = cumulativeTform * newFace.local(j);
			norm = vec3( cumulativeNormTform * vec4( norm, 0.0 ), VW );
			newFace.normal(j).normalize();
		} 

		if( fabs(newFace.local(0)[VY]-1.0) <= 0.000001 ) {
			newFace.texture(0)[VX] = 0.5*(newFace.texture(1)[VX] + newFace.texture(2)[VX]);
		} 

		newFace.plane() = computeTriPlaneEq( newFace.world(0), newFace.world(1), newFace.world(2) );
		tessellation.append(newFace);

	} else { 

		v12 = (v1 + v2).normalize();
		v23 = (v2 + v3).normalize();
		v31 = (v3 + v1).normalize();

		tessellateRec( tessellation, v1, v12, v31, level-1 );
		tessellateRec( tessellation, v2, v23, v12, level-1 );
		tessellateRec( tessellation, v3, v31, v23, level-1 );
		tessellateRec( tessellation, v12, v23, v31, level-1 );

	}
}

	

void Sphere::tessellate( List<Face>& tessellation )
{
	for( int i = 0; i < 24; i++ ) {
		int i1 = tindices[i][2];
		int i2 = tindices[i][1];
		int i3 = tindices[i][0];

		vec3 v1( vdata[i1][0], vdata[i1][1], vdata[i1][2] );
		vec3 v2( vdata[i2][0], vdata[i2][1], vdata[i2][2] );
		vec3 v3( vdata[i3][0], vdata[i3][1], vdata[i3][2] );

		tessellateRec( tessellation, v1, v2, v3, tessLevel );
	}
}
*/

int Sphere::faceCount()
{
	return 20 * (int)pow( 4, attrTessLevel() );
}


void Sphere::renderOGL()
{
	/*
	GLfloat mat[16];

	cumulativeTform.asFloat( mat );
	glPushMatrix();
	glMultMatrixf( mat );

	glutSolidSphere( 1.0, 20, 20 );

	glPopMatrix();
	*/
}
