#ifndef Texture_h
#define Texture_h

#include "NamedEntity.h"
#include "algebra3.h"
#include "List.h"

class Sample;

/****************************************************************

	Class:		Texture
	Purpose:	Class representing a texture.  A Texture must be
				able to compute either a scalar or a vector value.
				Textures always take their input from the static
				sample Attribute::curSample().

*****************************************************************/
class Texture : public NamedEntity
{
public:
	Texture();

	// attribute initialization
	virtual void initializeAttributes();

	// attribute indices
	static int kActive;		// indicates whether or not this texture is
							// enabled

	// attribute accessors
	bool attrActive() { return findAttr(kActive)->asBool(); };

	// compute methods
	virtual vec3 vector() = 0;
	virtual double scalar() = 0;

	// updates the texture transform at the current time
	virtual void updateTransform() = 0;

	// preprocess function called before any rendering is done
	virtual void preProcess() {};

	// postprocess function called after all rendering is done
	virtual void postProcess() {};

};

#endif