#include "Texture2D.h"

//
//	Attribute index definitions
//
int Texture2D::kTranslate;
int Texture2D::kRotate;
int Texture2D::kScale;

//
//  Constructor just initializes attributes for this instance
//
Texture2D::Texture2D()
{
	Texture2D::initializeAttributes();
}

//
//  Attribute initialization
//
void Texture2D::initializeAttributes()
{
	Texture::initializeAttributes();

	kTranslate = addAttr( new AttributeVec2( "translate", ANIMATABLE, vec2(0.0) ) );
	kRotate = addAttr( new AttributeDouble( "rotate", ANIMATABLE, 0.0 ) );
	kScale = addAttr( new AttributeVec2( "scale", ANIMATABLE, vec2(1.0) ) );
}

//
//	Computes the placement matrix from the translate, rotate, and scale
//  attribute values
//
void Texture2D::updateTransform()
{
	mat3 scaleMat = scaling2D( attrScale() );
	mat3 rotateMat = rotation2D( vec2(0.0), attrRotate() );
	mat3 translateMat = translation2D( attrTranslate() );

	fTransform = translateMat * rotateMat * scaleMat;
}


/*
// macro for indexing into texture array
#define INDEX(x,y,c) pixels[4*(interactiveRes*(y)+(x))+(c)]

//
//  Constructs a 2d texture map for use
//
void Texture2D::preProcessOGL()
{
	if( !attrActive() ) return;

	// get the interactive resolution
	int interactiveRes = attrOglResolution();

	// allocate the texture map array
	pixels = new GLfloat[interactiveRes*interactiveRes*4];
	
	// fill in the array
	for( int u = 0; u < interactiveRes; u++ ) {
		for( int v = 0; v < interactiveRes; v++ ) {

			Sample sample;

			double dU = (double)u / (double)interactiveRes;
			double dV = (double)v / (double)interactiveRes;

			sample.texCoords2d = tForm * vec2(dU,dV);

			vec3 result = vector( sample ).clamp(0.0,1.0);

			INDEX(u,v,0) = (GLfloat)(result[RED]);
			INDEX(u,v,1) = (GLfloat)(result[GREEN]);
			INDEX(u,v,2) = (GLfloat)(result[BLUE]);
			INDEX(u,v,3) = (GLfloat)0.0;
		}
	}
}

void Texture2D::setupOGL()
{
	glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
	glPixelStorei(GL_UNPACK_ALIGNMENT,4);
	glTexImage2D( GL_TEXTURE_2D, 0, 4, interactiveRes, interactiveRes, 
				  0, GL_RGBA, GL_FLOAT, pixels );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glEnable( GL_TEXTURE_2D );
}

void Texture2D::interactivePostProcess()
{
	if( pixels != NULL ) {
		delete [] pixels;
	}
}
*/
