#include "Texture3D.h"

//
//  Attribute index definitions
//
int Texture3D::kTranslate;
int Texture3D::kRotate;
int Texture3D::kScale;

//
//  Constructor just initializes attributes
//
Texture3D::Texture3D()
{
	Texture3D::initializeAttributes();
}

//
//  Attribute initialization
//
void Texture3D::initializeAttributes()
{
	Texture::initializeAttributes();

	kTranslate = addAttr( new AttributeVec3( "translate", ANIMATABLE, vec3(0.0) ) );
	kRotate = addAttr( new AttributeVec3( "rotate", ANIMATABLE, vec3(0.0) ) );
	kScale = addAttr( new AttributeVec3( "scale", ANIMATABLE, vec3(1.0) ) );
}

//
//  Computes the 3d texture transform
//
void Texture3D::updateTransform()
{
	// no parent
	mat4 scaleMat = scaling3D( attrScale() );

	vec3 rotateVec = attrRotate();

	mat4 rotateMat = rotation3D( vec3(1.0,0.0,0.0), rotateVec[VX] ) *
					 rotation3D( vec3(0.0,1.0,0.0), rotateVec[VY] ) *
					 rotation3D( vec3(0.0,0.0,1.0), rotateVec[VZ] );

	mat4 translateMat = translation3D( attrTranslate() );

	fTransform = translateMat * rotateMat * scaleMat;
}

