#ifndef TextureTools_h
#define TextureTools_h

//
//  TextyreTools.h provides some useful functions for textures
//


// floor/ceiling macros
#define FLOOR(x) (fastDblToInt(x) - ((x) < 0 && (x) != fastDblToInt(x)))
#define CEIL(x) (fastDblToInt(x) + ((x) > 0 && (x) != fastDblToInt(x)))

// cubic Catmull-Rom spline
#define CR00	-0.5
#define CR01	1.5
#define CR02	-1.5
#define CR03	0.5
#define CR10	1.0
#define CR11	-2.5
#define CR12	2.0
#define CR13	-0.5
#define CR20	-0.5
#define CR21	0.0
#define CR22	0.5
#define CR23	0.0
#define CR30	0.0
#define CR31	1.0	
#define CR32	0.0
#define CR33	0.0

//
//  Catmull-Rom spline interpolant
//
template <class T> T spline( double x, int nknots, T *knot )
{
	int span;
	int nspans = nknots - 3;
	T c0, c1, c2, c3;

	assert( nspans >= 1 );

	x = clamp(x,0.0,1.0)*nspans;
	span = fastDblToInt(floor(x));

	if( span >= nknots-3 ) {
		span = nknots - 3;
	}

	x -= span;

	knot += span;

	c3 = CR00*knot[0] + CR01*knot[1] + CR02*knot[2] + CR03*knot[3];
	c2 = CR10*knot[0] + CR11*knot[1] + CR12*knot[2] + CR13*knot[3];
	c1 = CR20*knot[0] + CR21*knot[1] + CR22*knot[2] + CR23*knot[3];
	c0 = CR30*knot[0] + CR31*knot[1] + CR32*knot[2] + CR33*knot[3];

	return ((c3*x + c2)*x + c1)*x + c0;
}



#endif