#ifndef Triangle_h
#define Triangle_h

#include "algebra3.h"
#include "iostream.h"
#include "List.h"

class Sample;
class TriageMask;
class Bitmask;

/****************************************************************

	Class:		Triangle
	Purpose:	Represents a triangle suitable for rendering.  Stores
				world/screen space positions, texture coords, and
				normals for all 3 vertices.

*****************************************************************/
class Triangle
{
public:
	Triangle() {};

	// data accessors
	vec3& world   (int i ) { return wPoints[i]; };
	vec3& screen  (int i ) { return sPoints[i]; };
	vec2& texture (int i ) { return tPoints[i]; };
	vec3& normal  (int i ) { return normals[i]; };

	// fills a shading sample for the specified vertex
	void fillSample( int whichVertex, Sample& sample );

	// fills a shading sample for a point expressed as an
	// affine combination of the 3 triangle vertices
	void fillSample( vec3& affineCoords, Sample& sample );

	// computes triage masks/bitmasks for this triangle
	TriageMask triageMask( vec4 screenBox, bool correct );
	Bitmask bitMask( vec4 screenBox, bool correct );

	// output operator
	friend ostream& operator<< (ostream& s, Triangle& t );

private:

	vec3 wPoints[3];	// world space
	vec3 sPoints[3];	// screen space
	vec2 tPoints[3];	// texture space
	vec3 normals[3];	// normals
};

#endif